// Copyright (C) 2022 - 2023, Xilinx Inc - All rights reserved
//
// Licensed under the Apache License, Version 2.0 (the "License"). You may
// not use this file except in compliance with the License. A copy of the
// License is located at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
// WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
// License for the specific language governing permissions and limitations
// under the License.

#include <iostream>
#include <xrm.h>
#include <boost/property_tree/ptree.hpp>
#include <boost/property_tree/json_parser.hpp>
#include "create_json.h"

namespace pt = boost::property_tree;

/**
 * @brief Write the ptree to the output char*
 * 
 * @param output Char* which will store the json output
 * @param tree Tree containing information to export
 */
inline static void write_json_to_string(std::string& output, const pt::ptree& tree) {
  std::stringstream json;
  pt::write_json(json, tree);
  output = json.str();
  return;
}

/**
 * @brief Create an input ptree object using the given attributes
 * 
 * @param xrm_props Contains relevant information to create json for load calc
 * @param func_name The name of the function: DECODER, SCALER, LOOKAHEAD, ENCODER
 * @return pt::ptree 
 */
static pt::ptree create_input_ptree(XrmInterfaceProperties* input_props, XrmInterfaceProperties* out_props, int num_outputs, const std::string& func_name, bool is_xav1) {
  pt::ptree   val_tree = {};
  std::string format;
  uint32_t    type = 1; // for XAV1 - default case
  if (is_xav1) {
    format = "AV1";
  } else {
    format = "H264"; // XRM is just a bookkeeper and don't care, even if it is VC8000E AV1 format
    type   = 2;      // Override, if explicitly specified as VC8000E AV1
  }
  val_tree.put("function", func_name);
  val_tree.put("format", format);
  if (func_name == "ENCODER") {
    val_tree.put("type", type);
    if (input_props->is_la_enabled) {
      val_tree.put("lookahead", input_props->is_la_enabled);
    }
  }
  val_tree.put("resolution.input.width", input_props->width);
  val_tree.put("resolution.input.height", input_props->height);
  val_tree.put("resolution.input.frame-rate.num", input_props->fps_num);
  val_tree.put("resolution.input.frame-rate.den", input_props->fps_den);

  pt::ptree out_tree = {};
  for (int i = 0; i < num_outputs; i++) {
    pt::ptree sub_tree = {};
    sub_tree.put("width", out_props[i].width);
    sub_tree.put("height", out_props[i].height);
    sub_tree.put("frame-rate.num", out_props[i].fps_num);
    sub_tree.put("frame-rate.den", out_props[i].fps_den);
    out_tree.push_back(std::make_pair("", sub_tree));
  }
  if (num_outputs > 0) {
    val_tree.add_child("resolution.output", out_tree);
  }
  pt::ptree res_tree = {};
  res_tree.push_back(std::make_pair("", val_tree));
  pt::ptree top_tree;
  top_tree.add_child("request.parameters.resources", res_tree);
  return top_tree;
}

/**
 * @brief Create a json object using the given attributes
 * 
 * @param xrm_props Properties relevant for creating a json for MA35
 * @param func_name The name of the function: DECODER, SCALER, LOOKAHEAD, ENCODER
 * @param output Where to store the resulting json
 */
void create_json(
    XrmInterfaceProperties* input_props, XrmInterfaceProperties* output_props, int num_outputs, const std::string& func_name, std::string& output, bool is_xav1) {
  if (func_name != "DECODER" && func_name != "ENCODER" && func_name != "SCALER") {
    std::cerr << "ERROR: \"" << func_name << "\" unrecognized!" << std::endl;
    return;
  }
  if (input_props->width == 0 || input_props->height == 0) {
    std::cerr << "ERROR: Can't create json for input " << input_props->width << "x" << input_props->height << "!" << std::endl;
  }
  if (func_name == "SCALER" && num_outputs <= 0) {
    std::cerr << "ERROR: Can't create json for scaler with no outputs!" << std::endl;
  }
  pt::ptree tree = create_input_ptree(input_props, output_props, num_outputs, func_name, is_xav1);
  write_json_to_string(output, tree);
}
